@@ -1,3 +1,4 @@
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*************************************************************************
  *
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
@@ -28,9 +29,7 @@
 // MARKER(update_precomp.py): autogen include statement, do not remove
 #include "precompiled_accessibility.hxx"
 
-#ifndef _TOOLKIT_AWT_VCLXACCESSIBLECOMPONENT_HXX_
 #include <accessibility/extended/textwindowaccessibility.hxx>
-#endif
 #include "comphelper/accessibleeventnotifier.hxx"
 #include "unotools/accessiblerelationsethelper.hxx"
 #include <unotools/accessiblestatesethelper.hxx>
@@ -2101,7 +2100,7 @@ void Document::handleSelectionChangeNotification()
     }
     else if ( aOldTextStart == aOldTextEnd && aNewTextStart == aNewTextEnd )
     {
-        // old an new selection empty => no events
+        // old and new selection empty => no events
         nFirst1 = 0;
         nLast1 = 0;
         nFirst2 = 0;
@@ -2252,3 +2251,4 @@ void Document::disposeParagraphs()
 
 }
 
+/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
