@@ -1,3 +1,4 @@
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*************************************************************************
  *
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
@@ -330,6 +331,8 @@ IMPL_LINK( SvBaseLinksDlg, UpdateNowClickHdl, PushButton *, EMPTYARG )
             rListBox.Select( pE );
             rListBox.MakeVisible( pE );
         }
+
+        pNewMgr->CloseCachedComps();
     }
     return 0;
 }
@@ -711,3 +714,4 @@ void SvBaseLinksDlg::SetActLink( SvBaseLink * pLink )
     }
 }
 
+/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
