@@ -1,7 +1,8 @@
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*************************************************************************
  *
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
- * 
+ *
  * Copyright 2000, 2010 Oracle and/or its affiliates.
  *
  * OpenOffice.org - a multi-platform office productivity suite
@@ -44,17 +45,13 @@
 #include <com/sun/star/lang/XMultiServiceFactory.hpp>
 #include <com/sun/star/ucb/XProgressHandler.hpp>
 
-#ifndef _CRC32_HXX_
 #include <CRC32.hxx>
-#endif
 
 #include <string.h> // for memcpy
 #include <vector>
 
 #include <comphelper/storagehelper.hxx>
 
-using namespace vos;
-using namespace rtl;
 using namespace com::sun::star;
 using namespace com::sun::star::io;
 using namespace com::sun::star::uno;
@@ -64,6 +61,7 @@ using namespace com::sun::star::packages;
 using namespace com::sun::star::packages::zip;
 using namespace com::sun::star::packages::zip::ZipConstants;
 
+using rtl::OUString;
 
 /** This class is used to read entries from a zip file
  */
@@ -126,10 +124,10 @@ void ZipFile::setInputStream ( Reference < XInputStream > xNewStream )
     aGrabber.setInputStream ( xStream );
 }
 
-sal_Bool ZipFile::StaticGetCipher ( const ORef < EncryptionData > & xEncryptionData, rtlCipher &rCipher, sal_Bool bDecode )
+sal_Bool ZipFile::StaticGetCipher ( const rtl::Reference < EncryptionData > & xEncryptionData, rtlCipher &rCipher, sal_Bool bDecode )
 {
     sal_Bool bResult = sal_False;
-    if ( ! xEncryptionData.isEmpty() )
+    if ( xEncryptionData.is() )
     {
         Sequence < sal_uInt8 > aDerivedKey (16);
         rtlCipherError aResult;
@@ -157,7 +155,7 @@ sal_Bool ZipFile::StaticGetCipher ( const ORef < EncryptionData > & xEncryptionD
     return bResult;
 }
 
-void ZipFile::StaticFillHeader ( const ORef < EncryptionData > & rData, 
+void ZipFile::StaticFillHeader ( const rtl::Reference < EncryptionData > & rData,
                                 sal_Int32 nSize,
                                 const ::rtl::OUString& aMediaType,
                                 sal_Int8 * & pHeader )
@@ -208,23 +206,23 @@ void ZipFile::StaticFillHeader ( const ORef < EncryptionData > & rData,
     *(pHeader++) = static_cast< sal_Int8 >(( nMediaTypeLength >> 8 ) & 0xFF);
 
     // Then the salt content
-    memcpy ( pHeader, rData->aSalt.getConstArray(), nSaltLength ); 
+    memcpy ( pHeader, rData->aSalt.getConstArray(), nSaltLength );
     pHeader += nSaltLength;
 
     // Then the IV content
-    memcpy ( pHeader, rData->aInitVector.getConstArray(), nIVLength ); 
+    memcpy ( pHeader, rData->aInitVector.getConstArray(), nIVLength );
     pHeader += nIVLength;
 
     // Then the digest content
-    memcpy ( pHeader, rData->aDigest.getConstArray(), nDigestLength ); 
+    memcpy ( pHeader, rData->aDigest.getConstArray(), nDigestLength );
     pHeader += nDigestLength;
 
     // Then the mediatype itself
-    memcpy ( pHeader, aMediaType.getStr(), nMediaTypeLength ); 
+    memcpy ( pHeader, aMediaType.getStr(), nMediaTypeLength );
     pHeader += nMediaTypeLength;
 }
 
-sal_Bool ZipFile::StaticFillData ( ORef < EncryptionData > & rData,
+sal_Bool ZipFile::StaticFillData ( rtl::Reference < EncryptionData > & rData,
                                     sal_Int32 &rSize,
                                     ::rtl::OUString& aMediaType,
                                     Reference < XInputStream > &rStream )
@@ -289,10 +287,10 @@ sal_Bool ZipFile::StaticFillData ( ORef < EncryptionData > & rData,
 }
 
 Reference< XInputStream > ZipFile::StaticGetDataFromRawStream(	const Reference< XInputStream >& xStream,
-                                                                const ORef < EncryptionData > &rData )
+                                                                const rtl::Reference < EncryptionData > &rData )
         throw ( packages::WrongPasswordException, ZipIOException, RuntimeException )
 {
-    if ( rData.isEmpty() )
+    if ( !rData.is() )
         throw ZipIOException( OUString::createFromAscii( "Encrypted stream without encryption data!\n" ),
                             Reference< XInterface >() );
 
@@ -314,14 +312,14 @@ Reference< XInputStream > ZipFile::StaticGetDataFromRawStream(	const Reference<
         nSize = nSize > n_ConstDigestLength ? n_ConstDigestLength : nSize;
 
         // skip header
-        xSeek->seek( n_ConstHeaderSize + rData->aInitVector.getLength() + 
+        xSeek->seek( n_ConstHeaderSize + rData->aInitVector.getLength() +
                                 rData->aSalt.getLength() + rData->aDigest.getLength() );
 
         // Only want to read enough to verify the digest
         Sequence < sal_Int8 > aReadBuffer ( nSize );
 
-        xStream->readBytes( aReadBuffer, nSize ); 
-    
+        xStream->readBytes( aReadBuffer, nSize );
+
         if ( !StaticHasValidPassword( aReadBuffer, rData ) )
             throw packages::WrongPasswordException( ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( OSL_LOG_PREFIX ) ), uno::Reference< uno::XInterface >() );
     }
@@ -329,9 +327,9 @@ Reference< XInputStream > ZipFile::StaticGetDataFromRawStream(	const Reference<
     return new XUnbufferedStream ( xStream, rData );
 }
 
-sal_Bool ZipFile::StaticHasValidPassword( const Sequence< sal_Int8 > &aReadBuffer, const ORef < EncryptionData > &rData )
+sal_Bool ZipFile::StaticHasValidPassword( const Sequence< sal_Int8 > &aReadBuffer, const rtl::Reference < EncryptionData > &rData )
 {
-    if ( !rData.isValid() || !rData->aKey.getLength() )
+    if ( !rData.is() || !rData->aKey.getLength() )
         return sal_False;
 
     sal_Bool bRet = sal_False;
@@ -353,7 +351,7 @@ sal_Bool ZipFile::StaticHasValidPassword( const Sequence< sal_Int8 > &aReadBuffe
     if(aResult != rtl_Cipher_E_None ) {
         OSL_ASSERT ( aResult == rtl_Cipher_E_None);
     }
-    
+
     aDigestResult = rtl_digest_updateSHA1 ( aDigest,
                                             static_cast < const void * > ( aDecryptBuffer.getConstArray() ), nSize );
     OSL_ASSERT ( aDigestResult == rtl_Digest_E_None );
@@ -362,10 +360,10 @@ sal_Bool ZipFile::StaticHasValidPassword( const Sequence< sal_Int8 > &aReadBuffe
     OSL_ASSERT ( aDigestResult == rtl_Digest_E_None );
 
     // If we don't have a digest, then we have to assume that the password is correct
-    if (  rData->aDigest.getLength() != 0  && 
+    if (  rData->aDigest.getLength() != 0  &&
           ( aDigestSeq.getLength() != rData->aDigest.getLength() ||
-            0 != rtl_compareMemory ( aDigestSeq.getConstArray(), 
-                                     rData->aDigest.getConstArray(), 
+            0 != rtl_compareMemory ( aDigestSeq.getConstArray(),
+                                     rData->aDigest.getConstArray(),
                                     aDigestSeq.getLength() ) ) )
     {
         // We should probably tell the user that the password they entered was wrong
@@ -378,7 +376,7 @@ sal_Bool ZipFile::StaticHasValidPassword( const Sequence< sal_Int8 > &aReadBuffe
     return bRet;
 }
 
-sal_Bool ZipFile::hasValidPassword ( ZipEntry & rEntry, const ORef < EncryptionData > &rData )
+sal_Bool ZipFile::hasValidPassword ( ZipEntry & rEntry, const rtl::Reference < EncryptionData > &rData )
 {
     ::osl::MutexGuard aGuard( m_aMutex );
 
@@ -392,7 +390,7 @@ sal_Bool ZipFile::hasValidPassword ( ZipEntry & rEntry, const ORef < EncryptionD
         nSize = nSize > n_ConstDigestLength ? n_ConstDigestLength : nSize;
         Sequence < sal_Int8 > aReadBuffer ( nSize );
 
-        xStream->readBytes( aReadBuffer, nSize ); 
+        xStream->readBytes( aReadBuffer, nSize );
 
         bRet = StaticHasValidPassword( aReadBuffer, rData );
     }
@@ -402,7 +400,7 @@ sal_Bool ZipFile::hasValidPassword ( ZipEntry & rEntry, const ORef < EncryptionD
 #if 0
 Reference < XInputStream > ZipFile::createFileStream(
             ZipEntry & rEntry,
-            const ORef < EncryptionData > &rData,
+            const rtl::Reference < EncryptionData > &rData,
             sal_Bool bRawStream,
             sal_Bool bIsEncrypted )
 {
@@ -412,8 +410,8 @@ Reference < XInputStream > ZipFile::createFileStream(
 }
 Reference < XInputStream > ZipFile::createMemoryStream(
             ZipEntry & rEntry,
-            const ORef < EncryptionData > &rData,
-            sal_Bool bRawStream, 
+            const rtl::Reference < EncryptionData > &rData,
+            sal_Bool bRawStream,
             sal_Bool bIsEncrypted )
 {
     sal_Int32 nUncompressedSize, nEnd;
@@ -434,8 +432,8 @@ Reference < XInputStream > ZipFile::createMemoryStream(
     // If the encryption key is zero, we need to return the raw stream. First check if
     // we have the salt. If we have the salt, then check if we have the encryption key
     // if not, return rawStream instead.
-    
-    sal_Bool bHaveEncryptData = ( !rData.isEmpty() && rData->aSalt.getLength() && rData->aInitVector.getLength() && rData->nIterationCount != 0 ) ? sal_True : sal_False;
+
+    sal_Bool bHaveEncryptData = ( rData.is() && rData->aSalt.getLength() && rData->aInitVector.getLength() && rData->nIterationCount != 0 ) ? sal_True : sal_False;
     sal_Bool bMustDecrypt = ( !bRawStream && bHaveEncryptData && bIsEncrypted ) ? sal_True : sal_False;
 
     if ( bMustDecrypt )
@@ -477,10 +475,10 @@ Reference < XInputStream > ZipFile::createMemoryStream(
         // data to the stream
 
         // Make a buffer big enough to hold both the header and the data itself
-        Sequence < sal_Int8 > aEncryptedDataHeader ( n_ConstHeaderSize + 
-                                                     rData->aInitVector.getLength() + 
-                                                     rData->aSalt.getLength() + 
-                                                     rData->aDigest.getLength() + 
+        Sequence < sal_Int8 > aEncryptedDataHeader ( n_ConstHeaderSize +
+                                                     rData->aInitVector.getLength() +
+                                                     rData->aSalt.getLength() +
+                                                     rData->aDigest.getLength() +
                                                      aWriteBuffer.getLength() );
         sal_Int8 * pHeader = aEncryptedDataHeader.getArray();
         StaticFillHeader ( rData, rEntry.nSize, pHeader );
@@ -495,7 +493,7 @@ Reference < XInputStream > ZipFile::createMemoryStream(
 Reference < XInputStream > ZipFile::createUnbufferedStream(
             SotMutexHolderRef aMutexHolder,
             ZipEntry & rEntry,
-            const ORef < EncryptionData > &rData,
+            const rtl::Reference < EncryptionData > &rData,
             sal_Int8 nStreamMode,
             sal_Bool bIsEncrypted,
             ::rtl::OUString aMediaType )
@@ -512,7 +510,7 @@ ZipEnumeration * SAL_CALL ZipFile::entries(  )
 }
 
 Reference< XInputStream > SAL_CALL ZipFile::getInputStream( ZipEntry& rEntry,
-        const vos::ORef < EncryptionData > &rData,
+        const rtl::Reference < EncryptionData > &rData,
         sal_Bool bIsEncrypted,
         SotMutexHolderRef aMutexHolder )
     throw(IOException, ZipException, RuntimeException)
@@ -522,14 +520,14 @@ Reference< XInputStream > SAL_CALL ZipFile::getInputStream( ZipEntry& rEntry,
     if ( rEntry.nOffset <= 0 )
         readLOC( rEntry );
 
-    // We want to return a rawStream if we either don't have a key or if the 
+    // We want to return a rawStream if we either don't have a key or if the
     // key is wrong
-    
+
     sal_Bool bNeedRawStream = rEntry.nMethod == STORED;
-    
+
     // if we have a digest, then this file is an encrypted one and we should
     // check if we can decrypt it or not
-    if ( bIsEncrypted && !rData.isEmpty() && rData->aDigest.getLength() )
+    if ( bIsEncrypted && rData.is() && rData->aDigest.getLength() )
         bNeedRawStream = !hasValidPassword ( rEntry, rData );
 
     return createUnbufferedStream ( aMutexHolder,
@@ -540,7 +538,7 @@ Reference< XInputStream > SAL_CALL ZipFile::getInputStream( ZipEntry& rEntry,
 }
 
 Reference< XInputStream > SAL_CALL ZipFile::getDataStream( ZipEntry& rEntry,
-        const vos::ORef < EncryptionData > &rData,
+        const rtl::Reference < EncryptionData > &rData,
         sal_Bool bIsEncrypted,
         SotMutexHolderRef aMutexHolder )
     throw ( packages::WrongPasswordException,
@@ -553,14 +551,14 @@ Reference< XInputStream > SAL_CALL ZipFile::getDataStream( ZipEntry& rEntry,
     if ( rEntry.nOffset <= 0 )
         readLOC( rEntry );
 
-    // An exception must be thrown in case stream is encrypted and 
+    // An exception must be thrown in case stream is encrypted and
     // there is no key or the key is wrong
     sal_Bool bNeedRawStream = sal_False;
     if ( bIsEncrypted )
     {
         // in case no digest is provided there is no way
         // to detect password correctness
-        if ( rData.isEmpty() )
+        if ( !rData.is() )
             throw ZipException( OUString::createFromAscii( "Encrypted stream without encryption data!\n" ),
                                 Reference< XInterface >() );
 
@@ -581,7 +579,7 @@ Reference< XInputStream > SAL_CALL ZipFile::getDataStream( ZipEntry& rEntry,
 }
 
 Reference< XInputStream > SAL_CALL ZipFile::getRawData( ZipEntry& rEntry,
-        const vos::ORef < EncryptionData > &rData,
+        const rtl::Reference < EncryptionData > &rData,
         sal_Bool bIsEncrypted,
         SotMutexHolderRef aMutexHolder )
     throw(IOException, ZipException, RuntimeException)
@@ -596,7 +594,7 @@ Reference< XInputStream > SAL_CALL ZipFile::getRawData( ZipEntry& rEntry,
 
 Reference< XInputStream > SAL_CALL ZipFile::getWrappedRawStream(
         ZipEntry& rEntry,
-        const vos::ORef < EncryptionData > &rData,
+        const rtl::Reference < EncryptionData > &rData,
         const ::rtl::OUString& aMediaType,
         SotMutexHolderRef aMutexHolder )
     throw ( packages::NoEncryptionException,
@@ -606,7 +604,7 @@ Reference< XInputStream > SAL_CALL ZipFile::getWrappedRawStream(
 {
     ::osl::MutexGuard aGuard( m_aMutex );
 
-    if ( rData.isEmpty() )
+    if ( !rData.is() )
         throw packages::NoEncryptionException( ::rtl::OUString( RTL_CONSTASCII_USTRINGPARAM( OSL_LOG_PREFIX ) ), uno::Reference< uno::XInterface >() );
 
     if ( rEntry.nOffset <= 0 )
@@ -640,28 +638,37 @@ sal_Bool ZipFile::readLOC( ZipEntry &rEntry )
     aGrabber >> nExtraLen;
     rEntry.nOffset = static_cast < sal_Int32 > (aGrabber.getPosition()) + nPathLen + nExtraLen;
 
-    // read always in UTF8, some tools seem not to set UTF8 bit
-    uno::Sequence < sal_Int8 > aNameBuffer( nPathLen );
-    sal_Int32 nRead = aGrabber.readBytes( aNameBuffer, nPathLen );
-    if ( nRead < aNameBuffer.getLength() )
-            aNameBuffer.realloc( nRead );
+    sal_Bool bBroken = sal_False;
 
-    ::rtl::OUString sLOCPath = rtl::OUString::intern( (sal_Char *) aNameBuffer.getArray(), 
-                                                        aNameBuffer.getLength(), 
-                                                        RTL_TEXTENCODING_UTF8 );
-
-    if ( rEntry.nPathLen == -1 ) // the file was created
+    try
     {
-        rEntry.nPathLen = nPathLen;
-        rEntry.sPath = sLOCPath;
-    }
+        // read always in UTF8, some tools seem not to set UTF8 bit
+        uno::Sequence < sal_Int8 > aNameBuffer( nPathLen );
+        sal_Int32 nRead = aGrabber.readBytes( aNameBuffer, nPathLen );
+        if ( nRead < aNameBuffer.getLength() )
+                aNameBuffer.realloc( nRead );
 
-    // the method can be reset for internal use so it is not checked
-    sal_Bool bBroken = rEntry.nVersion != nVersion
-                    || rEntry.nFlag != nFlag
-                    || rEntry.nTime != nTime
-                    || rEntry.nPathLen != nPathLen
-                    || !rEntry.sPath.equals( sLOCPath );
+        ::rtl::OUString sLOCPath = rtl::OUString::intern( (sal_Char *) aNameBuffer.getArray(),
+                                                            aNameBuffer.getLength(),
+                                                            RTL_TEXTENCODING_UTF8 );
+
+        if ( rEntry.nPathLen == -1 ) // the file was created
+        {
+            rEntry.nPathLen = nPathLen;
+            rEntry.sPath = sLOCPath;
+        }
+
+        // the method can be reset for internal use so it is not checked
+        bBroken = rEntry.nVersion != nVersion
+                        || rEntry.nFlag != nFlag
+                        || rEntry.nTime != nTime
+                        || rEntry.nPathLen != nPathLen
+                        || !rEntry.sPath.equals( sLOCPath );
+    }
+    catch(::std::bad_alloc &)
+    {
+        bBroken = sal_True;
+    }
 
     if ( bBroken && !bRecoveryMode )
         throw ZipIOException( OUString( RTL_CONSTASCII_USTRINGPARAM( "The stream seems to be broken!" ) ),
@@ -797,15 +804,15 @@ sal_Int32 ZipFile::readCEN()
                 throw ZipException( OUString( RTL_CONSTASCII_USTRINGPARAM ( "unexpected extra header info length") ), Reference < XInterface > () );
 
             // read always in UTF8, some tools seem not to set UTF8 bit
-            aEntry.sPath = rtl::OUString::intern ( (sal_Char *) aMemGrabber.getCurrentPos(), 
-                                                   aEntry.nPathLen, 
+            aEntry.sPath = rtl::OUString::intern ( (sal_Char *) aMemGrabber.getCurrentPos(),
+                                                   aEntry.nPathLen,
                                                    RTL_TEXTENCODING_UTF8 );
 
             if ( !::comphelper::OStorageHelper::IsValidZipEntryFileName( aEntry.sPath, sal_True ) )
                 throw ZipException( OUString( RTL_CONSTASCII_USTRINGPARAM ( "Zip entry has an invalid name.") ), Reference < XInterface > () );
 
             aMemGrabber.skipBytes( aEntry.nPathLen + aEntry.nExtraLen + nCommentLen );
-            aEntries[aEntry.sPath] = aEntry;	
+            aEntries[aEntry.sPath] = aEntry;
         }
 
         if (nCount != nTotal)
@@ -836,13 +843,18 @@ sal_Int32 ZipFile::recover()
 
         aGrabber.seek( 0 );
 
-        for( sal_Int32 nGenPos = 0; aGrabber.readBytes( aBuffer, 32000 ) && aBuffer.getLength() > 30; )
+        const sal_Int32 nToRead = 32000;
+        for( sal_Int32 nGenPos = 0; aGrabber.readBytes( aBuffer, nToRead ) && aBuffer.getLength() > 16; )
         {
             const sal_Int8 *pBuffer = aBuffer.getConstArray();
             sal_Int32 nBufSize = aBuffer.getLength();
 
             sal_Int32 nPos = 0;
-            while( nPos < nBufSize - 16 )
+            // the buffer should contain at least one header,
+            // or if it is end of the file, at least the postheader with sizes and hash
+            while( nPos < nBufSize - 30
+                || ( nBufSize < nToRead && nPos < nBufSize - 16 ) )
+
             {
                 if ( nPos < nBufSize - 30 && pBuffer[nPos] == 'P' && pBuffer[nPos+1] == 'K' && pBuffer[nPos+2] == 3 && pBuffer[nPos+3] == 4 )
                 {
@@ -864,7 +876,7 @@ sal_Int32 ZipFile::recover()
                             aMemGrabber >> aEntry.nPathLen;
                             aMemGrabber >> aEntry.nExtraLen;
 
-                            sal_Int32 nDescrLength = 
+                            sal_Int32 nDescrLength =
                                 ( aEntry.nMethod == DEFLATED && ( aEntry.nFlag & 8 ) ) ?
                                                         16 : 0;
 
@@ -884,16 +896,16 @@ sal_Int32 ZipFile::recover()
                             {
                                 // read always in UTF8, some tools seem not to set UTF8 bit
                                 if( nPos + 30 + aEntry.nPathLen <= nBufSize )
-                                    aEntry.sPath = OUString ( (sal_Char *) &pBuffer[nPos + 30], 
-                                                                  aEntry.nPathLen, 
+                                    aEntry.sPath = OUString ( (sal_Char *) &pBuffer[nPos + 30],
+                                                                  aEntry.nPathLen,
                                                                 RTL_TEXTENCODING_UTF8 );
                                 else
                                 {
                                     Sequence < sal_Int8 > aFileName;
                                     aGrabber.seek( nGenPos + nPos + 30 );
                                     aGrabber.readBytes( aFileName, aEntry.nPathLen );
-                                    aEntry.sPath = OUString ( (sal_Char *) aFileName.getArray(), 
-                                                                aFileName.getLength(), 
+                                    aEntry.sPath = OUString ( (sal_Char *) aFileName.getArray(),
+                                                                aFileName.getLength(),
                                                                 RTL_TEXTENCODING_UTF8 );
                                     aEntry.nPathLen = static_cast< sal_Int16 >(aFileName.getLength());
                                 }
@@ -1047,7 +1059,7 @@ void ZipFile::getSizeAndCRC( sal_Int32 nOffset, sal_Int32 nCompressedSize, sal_I
             aCRC.updateSegment( aData, 0, nLastInflated );
             nInBlock += nLastInflated;
         } while( !aInflater.finished() && nLastInflated );
-        
+
         nRealSize += nInBlock;
     }
 
@@ -1055,3 +1067,4 @@ void ZipFile::getSizeAndCRC( sal_Int32 nOffset, sal_Int32 nCompressedSize, sal_I
     *nCRC = aCRC.getValue();
 }
 
+/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
